<?php

namespace Drupal\eca_parameters\Plugin\ECA\Event;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eca\Attributes\Token;
use Drupal\eca\Entity\Objects\EcaEvent;
use Drupal\eca\Event\Tag;
use Drupal\eca\Plugin\ECA\Event\EventBase;
use Drupal\eca_parameters\Event\EcaParametersEvents;
use Drupal\eca_parameters\Event\RequestingParameterEvent;
use Symfony\Contracts\EventDispatcher\Event;

/**
 * Plugin implementation of the ECA base Events.
 *
 * @EcaEvent(
 *   id = "parameters",
 *   deriver = "Drupal\eca_parameters\Plugin\ECA\Event\ParametersEventDeriver",
 *   eca_version_introduced = "1.0.0"
 * )
 */
class ParametersEvent extends EventBase {

  /**
   * {@inheritdoc}
   */
  public static function definitions(): array {
    $definitions = [];
    $definitions['request'] = [
      'label' => 'Requesting parameter',
      'event_name' => EcaParametersEvents::REQUEST,
      'event_class' => RequestingParameterEvent::class,
      'tags' => Tag::READ | Tag::BEFORE | Tag::CONFIG,
    ];
    return $definitions;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    if ($this->eventClass() === RequestingParameterEvent::class) {
      $values = [
        'parameter_name' => '',
      ];
    }
    else {
      $values = [];
    }
    return $values + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    if ($this->eventClass() === RequestingParameterEvent::class) {
      $form['parameter_name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Parameter name'),
        '#description' => $this->t('The machine name of the requested parameter. Examples: <em>my_eca_parameter</em>, <em>my_global_param</em>, <em>my_article_param</em>.'),
        '#default_value' => $this->configuration['parameter_name'],
        '#required' => TRUE,
        '#weight' => -20,
      ];
    }
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    parent::submitConfigurationForm($form, $form_state);
    if ($this->eventClass() === RequestingParameterEvent::class) {
      $this->configuration['parameter_name'] = $form_state->getValue('parameter_name');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function generateWildcard(string $eca_config_id, EcaEvent $ecaEvent): string {
    return isset($this->configuration['parameter_name']) ? trim($this->configuration['parameter_name']) : '*';
  }

  /**
   * {@inheritdoc}
   */
  public static function appliesForWildcard(Event $event, string $event_name, string $wildcard): bool {
    if ($event instanceof RequestingParameterEvent && $wildcard === $event->getPreparationEvent()->name) {
      return TRUE;
    }
    if (($wildcard === '') || ($wildcard === '*')) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  #[Token(
    name: 'event',
    description: 'The event.',
    properties: [
      new Token(name: 'parameter_name', description: 'The parameter name.'),
      new Token(name: 'entity', description: 'The entity.'),
      new Token(name: 'ENTITY_TYPE', description: 'The entity type.'),
    ],
  )]
  protected function buildEventData(): array {
    $event = $this->event;
    $data = [];

    if ($event instanceof RequestingParameterEvent) {
      $data['parameter_name'] = $event->getPreparationEvent()->name;
      foreach ($event->getPreparationEvent()->context as $value) {
        if ($value instanceof EntityInterface) {
          $data['entity'] = $value;
          $data[$value->getEntityTypeId()] = $value;
        }
      }
    }

    $data += parent::buildEventData();
    return $data;
  }

}
